<?php
/**************************************************************************\
* eGroupWare                                                               *
* http://www.egroupware.org                                                *
* --------------------------------------------                             *
*  This program is free software; you can redistribute it and/or modify it *
*  under the terms of the GNU General Public License as published by the   *
*  Free Software Foundation; either version 2 of the License, or (at your  *
*  option) any later version.                                              *
\**************************************************************************/

require_once 'common.inc.php';

/**
 * Classe utilizada para protejer objetos do cdigo dos processos. Essa classe atua como um intermedirio entre o objeto protegido e o trecho de cdigo que tenta acessar um de seus mtodos/atributos.
 * @author Sidnei Augusto Drovetto Jr. - drovetto@gmail.com
 * @version 1.0
 * @package Workflow
 * @license http://www.gnu.org/copyleft/gpl.html GPL
 */
class WorkflowWatcher
{
	/**
	 * @var object $protectedObject O objeto que est sendo protegido
	 * @access private
	 */
	private $protectedObject;

	/**
	 * @var string $currentDir O diretrio a partir do qual, os arquivos de cdigo podem utilizar o objeto protegido
	 * @access private
	 */
	private $currentDir;

	/**
	 * @var int $currentDirLength O nmero de caracteres de $currentDir
	 * @access private
	 */
	private $currentDirLength;

	/**
	 * @var bool $enabledSecurity Indica se a segurana est ativa ou no
	 * @access private
	 */
	private static $enabledSecurity = false;

	/**
	 * Verifica se o arquivo que est acessando um mtodo/atributo do objeto possui permisso para tal
	 * @param int $index O ndice da pilha de chamadas de mtodos e funes que deve ser verificado para se encontrar a origem da chamada ao objeto protegido
	 * @return bool true caso o objeto possa ser acessado ou false caso contrrio
	 * @access private
	 */
	private final function workflowWatcherCheckSecurity($index = 2)
	{
		if (self::$enabledSecurity == false)
			return true;

		/* busca a pilha das chamadas de mtodos/funes */
		$debugBacktrace = debug_backtrace();

		/* checa se o acesso ao objeto ser permitido ou no */
		if ((!isset($debugBacktrace[$index]['file'])) || (substr($debugBacktrace[$index]['file'], 0, $this->currentDirLength) === $this->currentDir))
			return true;
		else
			return false;
	}

	/**
	 * Aplica as diretivas de segurana do mdulo
	 * @return void
	 * @access public
	 */
	public final function WorkflowWatcher($object)
	{
		$this->protectedObject = $object;
		$this->currentDir = dirname(__FILE__);
		$this->currentDir = substr($this->currentDir, 0, -13);
		$this->currentDirLength = strlen($this->currentDir);
	}

	/**
	 * Ativa a verificao de acesso ao objeto protegido. Por padro a segurana fica desabilitada pois ela s  interessante antes da execuo do cdigo dos processos e, sendo assim, s  habilitada nestes casos.
	 * @return void
	 * @access public
	 */
	public final static function workflowWatcherEnableSecurity()
	{
		self::$enabledSecurity = true;
	}

	/**
	 * Implementao do mtodo mgico "__call" que intercepta as chamadas ao objeto protegido
	 * @param string $methodName O nome do mtodo que est sendo requisitado
	 * @param array $arguments Os parmetros passados para o mtodo
	 * @return mixed O resultado da chamada do mtodo ou false se o cdigo que o mtodo no tiver permisso para tal
	 * @access public
	 */
	public final function __call($methodName, $arguments)
	{
		if (!$this->workflowWatcherCheckSecurity())
			return false;

		$method = new ReflectionMethod($this->protectedObject, $methodName);
		return $method->invokeArgs($this->protectedObject, $arguments);
	}

	/**
	 * Implementao do mtodo mgico "__set" que intercepta definies de atributos do objeto protegido
	 * @param string $name O nome do atributo ao qual est sendo atribudo um valor
	 * @param mixed $value O valor que ser atribudo ao atributo do objeto
	 * @return void
	 * @access public
	 */
	public final function __set($name, $value)
	{
		if (!$this->workflowWatcherCheckSecurity(1))
			return false;

		$this->protectedObject->{$name} = $value;
	}

	/**
	 * Implementao do mtodo mgico "__get" que intercepta leitura de atributos do objeto potegido
	 * @param string $name O nome do atributo que se est tentando acessar
	 * @return mixed O valor do atributo que se est acessando. Ou false, caso o acesso ao atributo seja negado
	 * @access public
	 */
	public final function __get($name)
	{
		if (!$this->workflowWatcherCheckSecurity(1))
			return false;

		return $this->protectedObject->{$name};
	}
}
?>
